/*
Connections:
  1. ESP8266 to OLED (I2C communication):
      OLED VCC → ESP8266 3.3V
      OLED GND → ESP8266 GND
      OLED SCL → ESP8266 D1 (GPIO5)
      OLED SDA → ESP8266 D2 (GPIO4)
  2. ESP8266 to BMP180 (I2C communication):
      BMP180 VCC → ESP8266 3.3V
      BMP180 GND → ESP8266 GND
      BMP180 SCL → ESP8266 D1 (GPIO5)
      BMP180 SDA → ESP8266 D2 (GPIO4)
  3. ESP8266 to NEO-6M GPS Module (UART communication):
      GPS VCC → ESP8266 3.3V
      GPS GND → ESP8266 GND
      GPS RX → ESP8266 D7 (GPIO13)
      GPS TX → ESP8266 D6 (GPIO12)
      
  created 20/10/2024
  by Akshaj Maheshwari
*/

#include <Adafruit_SSD1306.h>  // This library helps us control the OLED screen
#include <Adafruit_BMP085.h>   // This library helps us use the BMP180 sensor for altitude and pressure
#include <TinyGPS++.h>         // This library helps us get information from the GPS module
#include <TinyGPSPlus.h>       // An alternative name for the GPS library, used for compatibility
#include "SoftwareSerial.h"    // This library allows us to use additional serial (communication) pins

// Set up BMP180 sensor and GPS objects so we can use them later
Adafruit_BMP085 bmp;
TinyGPSPlus gps;

// Tell the microcontroller which pins we’re using for the GPS communication
SoftwareSerial ss(D3, D4);  // D3 (receive data from GPS), D4 (send data to GPS)

// OLED display configuration
#define SCREEN_WIDTH 128       // The width of the OLED display in pixels
#define SCREEN_HEIGHT 32       // The height of the OLED display in pixels
#define OLED_RESET -1          // Reset pin for the OLED display (we’re not using it here)
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);  // Setup OLED display object

void setup() {
  // Begin communication to our computer’s Serial Monitor
  Serial.begin(9600);
  // Begin communication with the GPS module
  ss.begin(9600);

  // Initialize the OLED display
  if (!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) {  // Check if the display starts correctly
    Serial.println(F("SSD1306 allocation failed"));   // Print error if display doesn’t work
    for (;;);                                        // Stop here if the display failed to start
  }
  display.clearDisplay();                            // Clear any random display data

  // Initialize the BMP180 sensor
  if (!bmp.begin()) {                                // Check if BMP180 sensor starts correctly
    Serial.println("Could not find a valid BMP180 sensor!");  // Print error if not working
    while (1) {}  // Stop here if the BMP180 isn’t found
  }

  // Show a welcome screen on the OLED display
  display.setTextSize(2);                  // Set big text size
  display.setTextColor(SSD1306_WHITE);     // Set text color to white
  display.setCursor(17, 0);                // Position text in the center horizontally
  display.println("NavAlert");             // Print the project name
  display.setTextSize(1);                  // Set smaller text size
  display.setCursor(10, 20);               // Position text below project name
  display.println("-AKSHAJ MAHESHWARI");   // Print author name
  display.display();                       // Show everything on the screen
  delay(2000);                             // Pause for 2 seconds to let people read
}

void loop() {
  // Get altitude data from BMP180 sensor
  int altitude = bmp.readAltitude();  // Altitude in meters

  // Clear the display before showing new data
  display.clearDisplay();
  display.setTextSize(1);  // Set text size to small for data display

  // Check if GPS data is available
  while (ss.available() > 0) {                  // If there’s data coming from the GPS
    if (gps.encode(ss.read())) {                // Process and decode it
      // Print GPS coordinates (latitude and longitude) on the Serial Monitor for debugging
      Serial.println(String(gps.location.lat(), 6) + "," + String(gps.location.lng(), 6));

      // Display Latitude
      display.setCursor(0, 0);                   // Position at top left corner
      display.print("LAT:");                     // Label for latitude
      display.setCursor(30, 0);                  // Position text next to the label
      display.print(gps.location.lat(), 6);      // Show the latitude with 6 decimal places

      // Display Longitude
      display.setCursor(0, 8);                   // Move cursor down a line
      display.print("LNG:");                     // Label for longitude
      display.setCursor(30, 8);
      display.print(gps.location.lng(), 6);      // Show longitude with 6 decimal places

      // Display Date information from GPS
      display.setCursor(90, 0);                  // Position on right side
      display.print("D:");                       // Day label
      display.println(gps.date.day());           // Show the day

      display.setCursor(90, 8);
      display.print("M:");                       // Month label
      display.println(gps.date.month());         // Show the month

      display.setCursor(90, 16);
      display.print("Y:");                       // Year label
      display.println(gps.date.year());          // Show the year

      // Get the GPS speed in meters per second (m/s)
      double speed_kmh = gps.speed.kmph();       // Speed in km/h
      double speed_mps = speed_kmh / 3.6;        // Convert km/h to m/s

      display.setCursor(60, 24);                 // Position cursor below date
      display.print("SPD:");                     // Label for speed
      display.print(speed_mps);                  // Show speed in m/s
      display.println("m/s");                    // Show units

      // Display Time from GPS if it’s valid (meaning there's signal)
      display.setCursor(0, 16);                  // Position cursor below coordinates
      if (gps.time.isValid()) {                  // If GPS has valid time data
        display.print("TIME:");                  // Label for time
        display.print(gps.time.hour());          // Show the hour
        display.print(":");
        if (gps.time.minute() < 10) display.print('0');  // Add leading zero for single digit minute
        display.print(gps.time.minute());
        display.print(":");
        if (gps.time.second() < 10) display.print('0');  // Add leading zero for single digit second
        display.print(gps.time.second());
      } else {
        display.println("Time: No signal");      // Show this if no GPS time data available
      }

      // Display Altitude (height above sea level)
      display.setCursor(0, 24);                  // Position cursor at the bottom left
      display.print("ALT:");                     // Label for altitude
      display.print(altitude);                   // Show altitude in meters
      display.println("m");                      // Show units

      // Show all the information we set on the screen
      display.display();
    }
  }
}